// SeeSaw platform tools module

#include <stdarg.h>
#include <errno.h>
#include <string.h>

#include "seesaw.h"
#include "i2cdevice.h"
#include "controls.h"

#define SEESAW_CHIP_DEFAULT_ADDRESS 0x2E

#define STATUS_BASE 0x00
#define STATUS_SWRST 0x7F
#define STATUS_RESET 0xff
#define READ_8_BUF_SIZE 1


typedef struct bytearray {
    unsigned char* array;
    size_t length;
} bytearray;


static bytearray* buildByteArray(va_list args, size_t n) {
    bytearray* barr = (bytearray*)malloc(sizeof(bytearray));
    barr->array = (unsigned char*)malloc(sizeof(unsigned char) * n);
    barr->length = n;

    for (int i = 0; i < n; i++) {
        barr->array[i] = va_arg(args, unsigned int);
    }

    return barr;
}

static void freeByteArray(bytearray* barr) {
    free(barr->array);
    free(barr);
}

void ssWrite(size_t n, ...) {
    va_list args;
    va_start(args, n);

    bytearray* barr = buildByteArray(args, n);

    va_end(args);
    int wbytes = i2cWrite(barr->array, barr->length);

    if (wbytes != (int)n) {
        printf("Error during i2cWrite: %s\n", strerror(errno));
    }

    freeByteArray(barr);
}

// cleans all the registers
// doesn't turn off LEDs though: have to do it manually
static void resetChip()  {
    ssWrite(3, STATUS_BASE, STATUS_SWRST, STATUS_RESET);
    sleepMS(500);
}

int ssRead(unsigned char base, unsigned char reg, unsigned char* buf, size_t n) {
    ssWrite(2, base, reg);
    int bytesRead = i2cRead(buf, n);

    return bytesRead;
}

unsigned char ssRead8(unsigned char base, unsigned char reg) {
    unsigned char buf[READ_8_BUF_SIZE];

    ssWrite(2, base, reg);
    int bytesRead = i2cRead(&buf, READ_8_BUF_SIZE);

    return buf[0];
}



void initializeSeeSawDevice(const char* bus, unsigned int deviceAddress) {
    if (deviceAddress != 0x00) {
        initializeI2C(bus, deviceAddress);
    } else {
        initializeI2C(bus, SEESAW_CHIP_DEFAULT_ADDRESS);
    }

    resetChip();
}

void destroySeeSawDevice() {
    destroyI2C();
}
