// SMBus protocol tool module

#include <stdlib.h>
#include <fcntl.h>
#include <stdio.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <linux/i2c-dev.h>

#include "smbus.h"


int i2c_bus;

void initializeSMBus(const char* bus) {
    i2c_bus = open(bus, O_RDWR);

    if (i2c_bus < 0) {
        printf("Error openning i2c-bus: %s\n", bus);
        exit(1);
    }
}

void closeSMBus() {
    close(i2c_bus);
}


static void setDeviceToSlaveMode(unsigned int deviceAddress) {
    if (ioctl(i2c_bus, I2C_SLAVE, deviceAddress & 0x7F) != 0) {
        printf("Error setting i2c-bus to slave mode\n");
    }
}

int writeTo(unsigned int deviceAddress, void* buffer, size_t len) {
    setDeviceToSlaveMode(deviceAddress);
    int bytesWritten = write(i2c_bus, buffer, len);
    return bytesWritten;
}

int readFrom(unsigned int deviceAddress, void* buf, size_t len) {
    setDeviceToSlaveMode(deviceAddress);
    int bytesRead = read(i2c_bus, buf, len);

    return bytesRead;
}
